package progressiveClientServer.receiveProxy;

import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import progressiveClientServer.receiveProxy.common.Constants;

import com.google.gson.Gson;
import com.google.gson.JsonIOException;
import com.google.gson.JsonSyntaxException;


public class ClientCommunicator {
//Singleton
	public static ClientCommunicator SINGLETON = new ClientCommunicator();
//Static variables
	private static Gson gson = new Gson();
	
//Attributes
	private String authorizationCode = "a1b2c3";
	
//Constructors
	public ClientCommunicator() {}
		
//Commands
	public int doubleValue(int value) {
		String doubleString = Integer.toString(value);
		HttpURLConnection connection =
		    //Example of how to embed the doubleString parameter value in the URI
			openConnection(Constants.DOUBLE_VALUE_DESIGNATOR + "/" + doubleString,
					       authorizationCode);
		int result = (int)returnResult(connection, int.class);

		return result;
	}
	
	public double negateValue() {		
		HttpURLConnection connection =
			openConnection(Constants.NEGATE_VALUE_DESIGNATOR,
						   authorizationCode);
		double result = (double)returnResult(connection, double.class);
	
		return result;
	}
	
	private HttpURLConnection openConnection(String contextIdentifier,
										     String authorizationCode) {
		HttpURLConnection result = null;
		try {
			URL url = new URL(URL_PREFIX + contextIdentifier);
			result = (HttpURLConnection)url.openConnection();
			result.setRequestMethod(HTTP_POST);
			result.setRequestProperty(Constants.AUTHORIZATION_KEY, authorizationCode);
			result.setDoOutput(false);
			result.connect();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return result;
	}
	
	private Object returnResult(HttpURLConnection connection, Class<?> klass) {
		Object result = null;
		try {
			if (connection.getResponseCode() == HttpURLConnection.HTTP_OK) {
				//0 means the body response length == 0; it was empty
				if(connection.getContentLength() == 0) {
					System.out.println("Response body was empty");
				} else if(connection.getContentLength() == -1) {
					//-1 means the body was not empty but has an unknown about of information
					InputStreamReader inputStreamReader = new InputStreamReader(connection.getInputStream());
					result = gson.fromJson(inputStreamReader, klass);
					inputStreamReader.close();
				}
			} else {
				throw new Exception(String.format("http code %d",	connection.getResponseCode()));
			}
		} catch (JsonSyntaxException | JsonIOException | IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result;
	}
		

//Auxiliary Constants, Attributes, and Methods
	private static final String SERVER_HOST = "localhost";
	private static final String URL_PREFIX = "http://" + SERVER_HOST + ":" + Constants.SERVER_PORT_NUMBER;
	private static final String HTTP_POST = "POST";
}
