package progressiveClientServer.sendProxy;


import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import progressiveClientServer.sendAndReceiveProxy.serverSideOnlyClasses.ServerCommunicator;

import com.google.gson.Gson;
import com.google.gson.JsonIOException;
import com.google.gson.JsonSyntaxException;

public class ClientCommunicator {
//Singleton
	public static ClientCommunicator SINGLETON = new ClientCommunicator();
//Static variables
	private static Gson gson = new Gson();
	
//Attributes
	
//Constructors
	public ClientCommunicator() {}
		
//Commands
	public void doubleValue(int value) {
		HttpURLConnection connection =
			openConnection(ServerCommunicator.DOUBLE_VALUE_DESIGNATOR, "a1b2c3");
		sendToServerCommunicator(connection, value);
		printResponseBodyInfo(connection);
	}
	
	public void negateValue(double value) {
		HttpURLConnection connection = openConnection(ServerCommunicator.NEGATE_VALUE_DESIGNATOR, "x9y8z7");
		sendToServerCommunicator(connection, value);
		printResponseBodyInfo(connection);
	}	
	
	private HttpURLConnection openConnection(String contextIdentifier, String authorizationCode) {
		HttpURLConnection result = null;
		try {
			URL url = new URL(URL_PREFIX + contextIdentifier);
			result = (HttpURLConnection)url.openConnection();
			result.setRequestMethod(HTTP_POST);
			result.setDoOutput(true);
			result.setRequestProperty(AUTHORIZATION_KEY, authorizationCode);
			result.connect();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return result;
	}
	
	private void sendToServerCommunicator(HttpURLConnection connection, Object objectToSend) {
		try {
			OutputStreamWriter outputStreamWriter = new OutputStreamWriter(connection.getOutputStream());
			gson.toJson(objectToSend, outputStreamWriter);
			outputStreamWriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void printResponseBodyInfo(HttpURLConnection connection) {
		try {
			if (connection.getResponseCode() == HttpURLConnection.HTTP_OK) {
				//0 means the body response length == 0; it was empty
				if(connection.getContentLength() == 0) {
					System.out.println("Response body was empty");
				} else {
					System.out.println("ERROR: The response body should have been empty");
				}
			} else {
				throw new Exception(String.format("http code %d",	connection.getResponseCode()));
			}
		} catch (JsonSyntaxException | JsonIOException | IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
		

//Auxiliary Constants, Attributes, and Methods
	private static final String SERVER_HOST = "localhost";
	private static final String URL_PREFIX = "http://" + SERVER_HOST + ":" + ServerCommunicator.getServerPortNumber();
	private static final String HTTP_POST = "POST";
	
	public static final String AUTHORIZATION_KEY = "authorization";

}
