package progressiveClientServer.sendingObjects;

import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;

import com.google.gson.Gson;
import com.sun.net.httpserver.Headers;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

public class ServerCommunicator {

	private static final int SERVER_PORT_NUMBER = 8080;
	private static final int MAX_WAITING_CONNECTIONS = 10;
	private static Gson gson = new Gson();
	
	private HttpServer server;
	
	private ServerCommunicator() {}

	private void run() {

		try {
			server = HttpServer.create(new InetSocketAddress(SERVER_PORT_NUMBER),
											MAX_WAITING_CONNECTIONS);
		} 
		catch (IOException e) {
			System.out.println("Could not create HTTP server: " + e.getMessage());
			e.printStackTrace();
			return;
		}

		server.setExecutor(null); // use the default executor
		
		server.createContext("/", sendingObjectsHandler);
		
		server.start();
	}
	
	private HttpHandler sendingObjectsHandler = new HttpHandler() {
		@Override
		public void handle(HttpExchange exchange) throws IOException {
			Headers headers = exchange.getRequestHeaders();
			String authorizationCode = headers.getFirst(ClientCommunicator.AUTHORIZATION_KEY);
			System.out.println("The authorization code in sendingObjectsHandler = " + authorizationCode);
			
			InputStreamReader inputStreamReader = new InputStreamReader(exchange.getRequestBody());
			Person person = (Person)gson.fromJson(inputStreamReader, Person.class);
			inputStreamReader.close();
			System.out.println("person = " + person);
			
			//-1 means the response body is empty
			exchange.sendResponseHeaders(HttpURLConnection.HTTP_OK, -1);
		}
	};
	
	public static int getServerPortNumber() {
		return SERVER_PORT_NUMBER;
	}
	
	public static void main(String[] args) {
		new ServerCommunicator().run();
	}

}
